package com.diagbot.facade;

import com.alibaba.fastjson.JSONArray;
import com.diagbot.client.ChiefPresentSimilarityServiceClient;
import com.diagbot.client.StandConvertServiceClient;
import com.diagbot.dto.BillNeoDTO;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.StandConvertCrfDTO;
import com.diagbot.entity.node.Disease;
import com.diagbot.entity.node.Medicine_Code;
import com.diagbot.entity.node.OralMedicine;
import com.diagbot.enums.StandConvertEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.model.ai.ModelAI;
import com.diagbot.repository.ICDDiseaseNode;
import com.diagbot.repository.ICDDiseaseRepository;
import com.diagbot.repository.DiseaseNode;
import com.diagbot.repository.DiseaseRepository;
import com.diagbot.repository.LISNode;
import com.diagbot.repository.LISPackNode;
import com.diagbot.repository.LisPackRepository;
import com.diagbot.repository.LisRepository;
import com.diagbot.repository.MedicineCodeRepository;
import com.diagbot.repository.MedicineNode;
import com.diagbot.repository.MedicineRepository;
import com.diagbot.repository.PACSNode;
import com.diagbot.repository.PacsRepository;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RedisUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.BillNeoVO;
import com.diagbot.vo.PushNeoVO;
import com.diagbot.vo.StandConvert;
import com.diagbot.vo.StandConvertCrfVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description: 图谱facade
 * @author: Mark
 * @time: 2018/8/6 9:11
 */
@Component
public class NeoFacade {

    @Autowired
    MedicineRepository medicineRepository;
    @Autowired
    PacsRepository pacsRepository;
    @Autowired
    LisRepository lisRepository;
    @Autowired
    LisPackRepository lisPackRepository;
    @Autowired
    DiseaseRepository diseaseRepository;
    @Autowired
    ICDDiseaseRepository icdDiseaseRepository;
    @Autowired
    MedicineCodeRepository medicineCodeRepository;
    @Autowired
    ChiefPresentSimilarityServiceClient chiefPresentSimilarityServiceClient;
    @Autowired
    NeoFacade self;
    @Autowired
    StandConvertServiceClient standConvertServiceClient;
    @Autowired
    RedisUtil redisUtil;

    /**
     * 返回药品缓存信息
     *
     * @return
     */
    @Cacheable(value = "cache", key = "'drugCache'")
    public Map<String, List<String>> getDrugCache() {
        Map<String, List<String>> res = new HashMap<>();

        Iterator<Medicine_Code> iterator = medicineCodeRepository.findAll().iterator();
        Medicine_Code current;
        String med_code, med_type;
        List<OralMedicine> sets;

        while (iterator.hasNext()) {
            current = iterator.next();
            if (current.getMedicine_related_chem().size()>0) {
                sets = new ArrayList<>(current.getMedicine_related_chem());
                med_code = current.getName();
                med_type = sets.get(0).getName();
                if (res.get(med_type)==null) {
                    res.put(med_type, new ArrayList<>());
                }
                if (res.get(med_type).indexOf(med_code)==-1) {
                    res.get(med_type).add(med_code);
                }
            }
        }

        // TODO 测试数据
        res.put("泰舒达类", Arrays.asList("泰舒达"));
        res.put("青霉素类", Arrays.asList("青霉素"));
        return res;
    }

    /**
     * 返回文本相似度匹配需要的词性列表
     *
     * @return
     */
    @Cacheable(value = "cache", key = "'similarCache'")
    public Map<String, List<String>> getSimilarCache() {
        Map<String, List<String>> res = new HashMap<>();
        // 临床表现，诊断，化验，辅检，药品，查体，手术
        res.put(StandConvertEnum.disease.getName(), getDiseaseList());
        // TODO 测试数据
        res.put(StandConvertEnum.clinical.getName(), Arrays.asList("胸痛","咳嗽"));
//        res.put(StandConvertEnum.disease.getName(), Arrays.asList("高血压", "糖尿病"));
        return res;
    }

    /**
     * 通过诊断反推
     *
     * @return
     */
    public Map<String, PushDTO> getInverse(PushNeoVO pushNeoVO) {
        Map<String, PushDTO> map = getDiag(pushNeoVO.getDiags());

        return map;
    }

    /**
     * 处理开单合理性图谱数据
     * @param billNeoVO
     * @return
     */
    public List<BillNeoDTO> getBillNeo(BillNeoVO billNeoVO) {

        List<BillNeoDTO> billNeoDTOs = new ArrayList<>();

        billNeoDTOs.addAll(getDrugBill(billNeoVO.getDrugList()));
        billNeoDTOs.addAll(getPacsBill(billNeoVO.getPacsList()));
        billNeoDTOs.addAll(getLisDetailBill(billNeoVO.getLisDetailList()));
        billNeoDTOs.addAll(getLisBill(billNeoVO.getLisList()));

        return billNeoDTOs;
    }



    /**
     * 处理处方开单合理性
     */
    public List<BillNeoDTO> getDrugBill(Map<String, String> medsterm) {

        MedicineNode medicineNode = new MedicineNode();
        List<BillNeoDTO> billNeoDTOs = medicineNode.getDrugBill(medsterm, medicineRepository);

        return billNeoDTOs;
    }


    /**
     * 处理辅检开单合理性
     */
    public List<BillNeoDTO> getPacsBill(Map<String, String> pacsterm) {

        PACSNode pacsNode = new PACSNode();
        List<BillNeoDTO> billNeoDTOs = pacsNode.getPacsBill(pacsterm, pacsRepository);

        return billNeoDTOs;
    }


    /**
     * 处理化验套餐开单合理性
     */
    public List<BillNeoDTO> getLisBill(Map<String, String> lispackterm) {

        LISPackNode lisPackNode = new LISPackNode();
        List<BillNeoDTO> billNeoDTOs = lisPackNode.getLisBill(lispackterm, lisPackRepository);

        return billNeoDTOs;
    }


    /**
     * 处理化验开单合理性
     */
    public List<BillNeoDTO> getLisDetailBill(Map<String, String> listerm) {

        LISNode lisNode = new LISNode();
        List<BillNeoDTO> billNeoDTOs = lisNode.getLisDetailBill(listerm, lisRepository);

        return billNeoDTOs;
    }


    public List<String> getDiseaseList() {

        ICDDiseaseNode diseaseNode = new ICDDiseaseNode();
        List<String> diseaselist = diseaseNode.getDiseaseNameList(icdDiseaseRepository);

        return diseaselist;
    }



    public Map<String, PushDTO> getDiag(List<String> diaglist) {

        Map<String, PushDTO> pushDTOs = new HashMap<>();
        PushDTO pushDTO;

        DiseaseNode diseaseNode = new DiseaseNode();
        Disease disease;

        for (String term : diaglist) {

            disease = diseaseRepository.findByNameIs(term);

            if (disease != null) {
                pushDTO = diseaseNode.DiseasetoDiseaseDTO(disease);

                pushDTOs.put(term, pushDTO);
            }
        }

        return pushDTOs;
    }


    /**
     * 标准词转换
     *
     * @param standConvert
     *
     * @return  Map<String, Map<String, String>> -->Map<类型, Map<原始词, 标准词>>
     */
    public Map<String, Map<String, String>> standConvert(StandConvert standConvert) {
        Map<String, Map<String, String>> map = new LinkedHashMap<>();
        Map<String, List<String>> similarMap = self.getSimilarCache();
        /* 文本相似度模型 */
        ModelAI modelAI = new ModelAI();

        // 临床表现数据
        List<String> list = standConvert.getClinicalList();
        Map<String, String> clinicalMap = getStandConvertTypeMap(modelAI, list,
                similarMap.get(StandConvertEnum.clinical.getName()), "chief_present");
        map.put(StandConvertEnum.clinical.getName(), clinicalMap);

        // 诊断数据
        List<String> diag_list = standConvert.getDiaglList();
        Map<String, String> diseaseMap = getStandConvertTypeMap(modelAI, diag_list,
                similarMap.get(StandConvertEnum.disease.getName()), "diagnose");
        map.put(StandConvertEnum.disease.getName(), diseaseMap);

        // 手术数据
        List<String> operation_list = standConvert.getOperationList();
        Map<String, String> operationMap = getStandConvertTypeMap(modelAI, operation_list,
                similarMap.get(StandConvertEnum.operation.getName()), "operation");
        map.put(StandConvertEnum.operation.getName(), operationMap);

        return map;
    }

    /**
     * 获取指定类型的map数据
     *
     * @param modelAI
     * @param text 文本内容
     * @param cacheList 模型列表
     * @param modelName 模块名称
     * @return
     */
    public Map<String, String> getStandConvertTypeMap(ModelAI modelAI, List<String> text, List<String> cacheList, String modelName) {
        Map<String, String> map = new LinkedHashMap<>();
        if (ListUtil.isEmpty(cacheList)) {
            return map;
        }
        for (String s : text) {
            JSONArray jsonArray = modelAI.loadSimilarAI(s, cacheList,
                    false, modelName, chiefPresentSimilarityServiceClient);
            putMap(jsonArray, s, map);
        }
        return map;
    }

    /**
     * 提取方法
     * @param jsonArray
     * @param originName
     * @param map
     */
    public void putMap(JSONArray jsonArray, String originName, Map<String, String> map) {
        if (jsonArray.size() == 2) {
            double likeRate = jsonArray.getDoubleValue(1);
            // System.out.println(originName + " " + likeRate);
            if (likeRate > 0.9) {
                String standName = jsonArray.getString(0);
                map.put(originName, standName);
            } else {
                map.put(originName, originName);
            }
        }
    }


    /**
     * 标准词转换
     * 类型，疾病: disease，症状: symptom，手术和操作:operation，药品: drug，实验室检查：lis，辅助检查：pacs， 辅助检查：vital"
     *
     * @param standConvert
     * @return  Map<String, Map<String, String>> -->Map<类型, Map<原始词, 标准词>>
     */
    public Map<String, Map<String, String>> standConvertCrf(StandConvert standConvert) {
        Map<String, Map<String, String>> map = new LinkedHashMap<>();

        try {
            List<String> clinicalList = standConvert.getClinicalList();
            Map<String, String> clinicalMap = getStandConvertMap(clinicalList, "symptom");
            map.put(StandConvertEnum.clinical.getName(), clinicalMap);

            List<String> operationList = standConvert.getOperationList();
            Map<String, String> operationMap = getStandConvertMap(operationList, "operation");
            map.put(StandConvertEnum.operation.getName(), operationMap);

            List<String> drugList = standConvert.getDrugList();
            Map<String, String> drugMap = getStandConvertMap(drugList, "drug");
            map.put(StandConvertEnum.drug.getName(), drugMap);

            List<String> vitalList = standConvert.getDrugList();
            Map<String, String> vitalMap = getStandConvertMap(vitalList, "vital");
            map.put(StandConvertEnum.vital.getName(), vitalMap);

            List<String> diseaseList = standConvert.getDiaglList();
            Map<String, String> diseaseMap = getStandConvertMap(diseaseList, "disease");
            map.put(StandConvertEnum.disease.getName(), diseaseMap);

        } catch (Exception e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "标准词转换服务出错!【找毕金良】" + e.getMessage());
        }
        return map;
    }


    /**
     * 标准词转换
     *
     * @param list 原词
     * @param type 类型
     * @return
     */
    public Map<String, String> getStandConvertMap(List<String> list, String type) {
        // long l1 = System.currentTimeMillis();
        Map<String, String> typeMap = redisUtil.getValueByType(type);
        Map<String, String> standConvertMap = new LinkedHashMap<>();
        List<String> notList = new ArrayList<>();

        if (typeMap != null) {
            for (String s : list) {
                if (StringUtil.isBlank(typeMap.get(s))) {
                    notList.add(s);
                }
            }
        } else {
            typeMap = new LinkedHashMap<>();
            notList = list;
        }

        if (ListUtil.isNotEmpty(notList)) {
            for (String s : notList) {
                StandConvertCrfVO standConvertCrfVO = new StandConvertCrfVO();
                standConvertCrfVO.setWord_type(type);
                standConvertCrfVO.setWord(s);
                StandConvertCrfDTO standConvertCrfDTO = standConvertServiceClient.similarity(standConvertCrfVO);
                if (ListUtil.isEmpty(standConvertCrfDTO.getStandard_words())) {
                    standConvertMap.put(s, s);
                } else {
                    standConvertMap.put(s, standConvertCrfDTO.getStandard_words().get(0).get("standard_word"));
                }
            }

            if (standConvertMap != null && !standConvertMap.isEmpty()) {
                typeMap.putAll(standConvertMap);
            }
            // long l2 = System.currentTimeMillis();
            // System.out.println("存放前" + type + ":" + (l2 - l1) / 1000.0);
            redisUtil.updateValueByType(typeMap, type);
            // long l3 = System.currentTimeMillis();
            // System.out.println("存放" + type + ":" + (l3 - l2) / 1000.0);
        }
        // long l4 = System.currentTimeMillis();
        // System.out.println("共耗时：" + type + ":" + (l4 - l1) / 1000.0);
        return typeMap;
    }
}
